# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2013
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################
"""
parsing des delas provenant des XML AAF
"""
from lxml import etree # fast and furious xml library

from eoleaaf.aaf import classes_for_one_eleve, classes_buffer, groupes_buffer, \
                eleves_responsable_buffer, sqlgen, parse_aaf_eleves, parse_aaf_responsables, \
                parse_aaf_profs, parse_aaf_etab_educnat, parse_aaf_subjects
from eoleaaf.util import pwgen, modif_fields, db, parse_multi, \
        get_new_id, gen_user_login
from eoleaaf.miscutil import user_id, groupe_eleve

from logging import getLogger
aaflog = getLogger('AAF')

# ______________________________________________________________________________
# eleves


def delete_request(filename, aaftype, parsetype):
    """delta correspondant a la suppression

    :param filename: fichier XML delta
    """
    aaflog.info("Deltas -- suppression des {}s...".format(aaftype))
    nbuser = 0
    context = etree.iterparse(filename, events=('end',), tag='deleteRequest')
    for event, xmldata in context:
        _type = xmldata.find('operationalAttributes/attr/value').text
        if _type != parsetype:
            aaflog.error("devrait être un {} et non un {} ".format(parsetype, _type))
            continue
        #**Attention** identifier/id == ENTPersonJointure
        dataitem = user_id(xmldata)
        # il faut aller voir dans la base si les champs n'existent pas
        db_item = db.fetchone(aaftype, 'id', "user", 'id', dataitem['id'], sqlgen)
        if db_item in ([], None):
            aaflog.error("ATTENTION: le {} {} censé être supprimé "
            "n'existe pas dans la base".format(aaftype, dataitem['id']))
            continue
        # ____________________________________________________________
        # suppression effective
        try:
            modif_fields(dataitem, sqlgen, 'DELETE', modif_type="DELTADELETE", tablename=aaftype, fromxml=True,
                         db=db)
            nbuser += 1
        except Exception as msg:
            aaflog.error("Erreur au moment de la suppression"
                    " du {} {} : {}".format(aaftype, str(dataitem['id']), str(msg) ))
        xmldata.clear()
        while xmldata.getprevious() is not None:
            del xmldata.getparent()[0]
    db.commit('user')
    aaflog.info("TOTAL : {} {}s".format(nbuser, aaftype))


def parse_delta_eleves(eleve_file):
    """parsing global des deltas pour les eleves

    :param eleve_file: fichier XML des eleves
    """
    aaflog.info("Lecture des delta des élèves...")
    # <addRequest> <modifyRequest> <deleteRequest>
    aaflog.info('ajout eleve')
    parse_aaf_eleves(eleve_file, modif_type='DELTACREATE')
    aaflog.info('modification eleve')
    #modify_request_eleve(eleve_file)
    parse_aaf_eleves(eleve_file, modif_type='DELTAUPDATE', parse_type='MODIFY')
    aaflog.info('suppression eleve')
    delete_request(eleve_file, aaftype="eleve", parsetype="Eleve")

# ______________________________________________________________________________
# responsables
#
def parse_delta_responsables(responsables_file):
    """Parsing global XML des responsables depuis AAF

    :param responsables_file: XML file name like ``*_PersRelEleve_*.xml``
    """
    aaflog.info("Lecture des responsables...")
    # <addRequest> <modifyRequest> <deleteRequest>
    aaflog.info('ajout responsable')
    parse_aaf_responsables(responsables_file, modif_type='DELTACREATE')
    aaflog.info('modification responsable')
    parse_aaf_responsables(responsables_file, modif_type='DELTAUPDATE', parse_type='MODIFY')
    aaflog.info('suppression responsable')
    delete_request(responsables_file, aaftype="responsable", parsetype="PersRelEleve")


def parse_delta_profs(profs_file):
    """ parsing des professeurs depuis AAF
    Extraction AAF Professeurs + Administratifs
    Fichier de type : ``ENT_$RNE_Complet_$DATE_PersEducNat_0000.xml``
    """
    aaflog.info("Lecture des deltas pour les personnels...")
    # <addRequest> <modifyRequest> <deleteRequest>
    aaflog.info('ajout professeur')
    parse_aaf_profs(profs_file, modif_type='DELTACREATE')
    aaflog.info('modification professeur')
    parse_aaf_profs(profs_file, modif_type='DELTAUPDATE', parse_type='MODIFY')
    aaflog.info('suppression professeur')
    delete_request(profs_file, aaftype="enseignant", parsetype="PersEducNat")
    delete_request(profs_file, aaftype="administratif", parsetype="PersEducNat")

# ____________________________________________________________
# etablissements

def parse_delta_etab_educnat(etab_file):
    """Parsing XML des responsables depuis AAF
    :responsables_file: file name like ``*_PersRelEleve_*.xml``
    """
    aaflog.info("Lecture des etablissements...")
    aaflog.info('ajout etablissement')
    # add_request_etab_educnat(etab_file)
    parse_aaf_etab_educnat(etab_file, modif_type='DELTACREATE')
    aaflog.info('modification etablissement')
    # modify_request_etab_educnat(etab_file)
    parse_aaf_etab_educnat(etab_file, modif_type='DELTAUPDATE', parse_type='MODIFY')


def parse_delta_subjects(subject_file):
    """parsing global des deltas pour les subjects

    :param eleve_file: fichier XML des eleves
    """
    aaflog.info("Lecture des delta des matieres...")
    # <addRequest> <modifyRequest> <deleteRequest>
    aaflog.info('ajout matiere')
    parse_aaf_subjects(subject_file, modif_type='DELTACREATE')
    aaflog.info('modification matiere')
    parse_aaf_subjects(subject_file, modif_type='DELTAUPDATE', parse_type='MODIFY')
    aaflog.info('suppression matiere')
    #delete_request_eleve(subject_file)
